/*----------------------------------------------------------------------------*/

/* VERS */

/*
  This file is part of Code_Saturne, a general-purpose CFD tool.

  Copyright (C) 1998-2014 EDF S.A.

  This program is free software; you can redistribute it and/or modify it under
  the terms of the GNU General Public License as published by the Free Software
  Foundation; either version 2 of the License, or (at your option) any later
  version.

  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
  details.

  You should have received a copy of the GNU General Public License along with
  this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
  Street, Fifth Floor, Boston, MA 02110-1301, USA.
*/

/*----------------------------------------------------------------------------*/

#include "cs_defs.h"

/*----------------------------------------------------------------------------
 * Standard C library headers
 *----------------------------------------------------------------------------*/

#include <assert.h>
#include <math.h>
#include <string.h>

#if defined(HAVE_MPI)
#include <mpi.h>
#endif

/*----------------------------------------------------------------------------
 * PLE library headers
 *----------------------------------------------------------------------------*/

#include <ple_coupling.h>

/*----------------------------------------------------------------------------
 *  Local headers
 *----------------------------------------------------------------------------*/

#include "bft_mem.h"
#include "bft_error.h"
#include "bft_printf.h"

#include "fvm_writer.h"

#include "cs_base.h"
#include "cs_field.h"
#include "cs_gui_util.h"
#include "cs_field_pointer.h"
#include "cs_field_operator.h"
#include "cs_math.h"
#include "cs_mesh.h"
#include "cs_mesh_location.h"
#include "cs_mesh_quantities.h"
#include "cs_halo.h"
#include "cs_halo_perio.h"
#include "cs_log.h"
#include "cs_multigrid.h"
#include "cs_parameters.h"
#include "cs_physical_constants.h"
#include "cs_prototypes.h"
#include "cs_sles.h"
#include "cs_sles_it.h"
#include "cs_time_moment.h"
#include "cs_time_step.h"
#include "cs_turbomachinery.h"
#include "cs_selector.h"

#include "cs_post.h"

/*----------------------------------------------------------------------------
 *  Header for the current file
 *----------------------------------------------------------------------------*/

#include "prototypes.h"

void
fvca_source(cs_real_t   f[],
            cs_real_6_t v[],
            cs_real_t   a[])
{

  const cs_lnum_t n_cells = cs_glob_mesh->n_cells;
  const cs_real_3_t *cell_cen = (const cs_real_3_t *)cs_glob_mesh_quantities->cell_cen;
  cs_real_t pi = acos(-1);
  cs_real_t x, y, z, solux, soluy, soluz, cxy, cxz, cz, sxy;
  cs_real_t sxz, sz, tpi, soluxx, soluyy, soluzz, soluxy, soluxz;
  cs_real_t soluyz, sx, sy, cx, cy;
  tpi = 2.*pi;

  /*----------------------------------------------------*/

  switch(testcase) {

    /*----------------------------------------------------*/
    case 1:
    for (cs_lnum_t iel = 0 ; iel < n_cells ; iel++) {
      x = cell_cen[iel][0];
      y = cell_cen[iel][1];
      z = cell_cen[iel][2];

      f[iel] = (v[iel][0] + v[iel][1] + v[iel][2])*pow(pi,2)
             * sin(pi * x)
             * sin(pi * (y + 0.5))
             * sin(pi * (z + 1./3.))
             - v[iel][3]*2*pow(pi,2)
             * cos(pi * x)
             * cos(pi * (y + 0.5))
             * sin(pi * (z + 1./3.))
             - v[iel][4]*2*pow(pi,2)
             * sin(pi * x)
             * cos(pi * (y + 0.5))
             * cos(pi * (z + 1./3.))
             - v[iel][5]*2*pow(pi,2)
             * cos(pi * x)
             * sin(pi * (y + 0.5))
             * cos(pi * (z + 1./3.));
    }
    break;

    /*----------------------------------------------------*/

    case 2:
    for (cs_lnum_t iel = 0 ; iel < n_cells ; iel++) {
      x = cell_cen[iel][0];
      y = cell_cen[iel][1];
      z = cell_cen[iel][2];

      sxz = sin(tpi*x*z);
      sxy = sin(tpi*x*y);
      sz = sin(tpi*z);

      cxz = cos(tpi*x*z);
      cxy = cos(tpi*x*y);
      cz = cos(tpi*z);

      solux = 3.*x*x*y*y*z + (sxz*sxy + x*z*tpi*cxz*sxy + x*y*tpi*sxz*cxy)*sz;
      soluy = x*x*x*2.*y*z + x*tpi*x*sxz*cxy*sz;
      soluz = x*x*x*y*y + x*sxy*tpi*(x*cxz*sz + sxz*cz);

      soluxx = 6.*x*y*y*z + ( 2.*tpi*z*cxz*sxy
                            + 2.*tpi*y*sxz*cxy
                            - x*(z*z + y*y)*tpi*tpi*sxz*sxy
                            + 2.*x*y*z*tpi*tpi*cxz*cxy ) * sz;

      soluyy = x*x*x*2.*z - x*tpi*x*sxz*tpi*x*sxy*sz;

      soluzz = x*sxy*tpi*tpi*(2.*x*cxz*cz - (x*x+1.)*sxz*sz);

      soluxy = 6.*x*x*y*z + ( tpi*x*sxz*cxy
                          + x*z*tpi*cxz*tpi*x*cxy
                          + x*tpi*sxz*cxy
                          - x*y*tpi*sxz*tpi*x*sxy ) * sz;

      soluxz = 3.*x*x*y*y + ( tpi*x*cxz*sxy
                          + x*tpi*cxz*sxy
                          - x*z*tpi*tpi*x*sxz*sxy
                          + x*y*tpi*tpi*x*cxz*cxy ) * sz
                          + tpi*(sxz*sxy + x*z*tpi*cxz*sxy
                          + x*y*tpi*sxz*cxy)*cz;

      soluyz = x*x*x*2.*y + x*tpi*x*cxy*tpi*(x*cxz*sz + sxz*cz);

      f[iel] = - ((pow(y,2)+pow(z,2)+1.)*soluxx
               +(pow(x,2)+pow(z,2)+1.)*soluyy
               +(pow(x,2)+pow(y,2)+1.)*soluzz
               - 2.*x*y*soluxy
               - 2.*x*z*soluxz
               - 2.*y*z*soluyz
               - soluy*y - soluz*z - solux*x -soluz*z - solux*x -soluy*y);

    }
    break;

    /*----------------------------------------------------*/

    case 3:
    for (cs_lnum_t iel = 0 ; iel < n_cells ; iel++) {
      x = cell_cen[iel][0];
      y = cell_cen[iel][1];
      z = cell_cen[iel][2];

      f[iel] = (v[iel][0] + v[iel][1] + v[iel][2])*4*pow(pi,2)
             * sin(tpi * x)
             * sin(tpi * y)
             * sin(tpi * z)
             - v[iel][3]*8*pow(pi,2)
             * cos(tpi * x)
             * cos(tpi * y)
             * sin(tpi * z)
             - v[iel][4]*8*pow(pi,2)
             * sin(tpi * x)
             * cos(tpi * y)
             * cos(tpi * z)
             - v[iel][5]*8*pow(pi,2)
             * cos(tpi * x)
             * sin(tpi * y)
             * cos(tpi * z);
    }
    break;

    /*----------------------------------------------------*/

    case 4:
    /* FIXME */
    for (cs_lnum_t iel = 0 ; iel < n_cells ; iel++) {
      f[iel] = 0.;
    }
    break;

    /*----------------------------------------------------*/

    case 5:
    for (cs_lnum_t iel = 0 ; iel < n_cells ; iel++) {
      x = cell_cen[iel][0];
      y = cell_cen[iel][1];
      z = cell_cen[iel][2];

      if (cell_cen[iel][1] <= 0.5 && cell_cen[iel][2] <= 0.5) {
      f[iel] = a[0] * (v[iel][0] + v[iel][1] + v[iel][2])*4*pow(pi,2)
                    * sin(tpi * x)
                    * sin(tpi * y)
                    * sin(tpi * z)
                    - v[iel][3]*8*pow(pi,2)
                    * cos(tpi * x)
                    * cos(tpi * y)
                    * sin(tpi * z)
                    - v[iel][4]*8*pow(pi,2)
                    * sin(tpi * x)
                    * cos(tpi * y)
                    * cos(tpi * z)
                    - v[iel][5]*8*pow(pi,2)
                    * cos(tpi * x)
                    * sin(tpi * y)
                    * cos(tpi * z);
      } else if (cell_cen[iel][1] > 0.5 && cell_cen[iel][2] <= 0.5) {
      f[iel] = a[1] * (v[iel][0] + v[iel][1] + v[iel][2])*4*pow(pi,2)
                    * sin(tpi * x)
                    * sin(tpi * y)
                    * sin(tpi * z)
                    - v[iel][3]*8*pow(pi,2)
                    * cos(tpi * x)
                    * cos(tpi * y)
                    * sin(tpi * z)
                    - v[iel][4]*8*pow(pi,2)
                    * sin(tpi * x)
                    * cos(tpi * y)
                    * cos(tpi * z)
                    - v[iel][5]*8*pow(pi,2)
                    * cos(tpi * x)
                    * sin(tpi * y)
                    * cos(tpi * z);
      } else if (cell_cen[iel][1] > 0.5 && cell_cen[iel][2] > 0.5) {
      f[iel] = a[2] * (v[iel][0] + v[iel][1] + v[iel][2])*4*pow(pi,2)
                    * sin(tpi * x)
                    * sin(tpi * y)
                    * sin(tpi * z)
                    - v[iel][3]*8*pow(pi,2)
                    * cos(tpi * x)
                    * cos(tpi * y)
                    * sin(tpi * z)
                    - v[iel][4]*8*pow(pi,2)
                    * sin(tpi * x)
                    * cos(tpi * y)
                    * cos(tpi * z)
                    - v[iel][5]*8*pow(pi,2)
                    * cos(tpi * x)
                    * sin(tpi * y)
                    * cos(tpi * z);
      } else {
      f[iel] = a[3] * (v[iel][0] + v[iel][1] + v[iel][2])*4*pow(pi,2)
                    * sin(tpi * x)
                    * sin(tpi * y)
                    * sin(tpi * z)
                    - v[iel][3]*8*pow(pi,2)
                    * cos(tpi * x)
                    * cos(tpi * y)
                    * sin(tpi * z)
                    - v[iel][4]*8*pow(pi,2)
                    * sin(tpi * x)
                    * cos(tpi * y)
                    * cos(tpi * z)
                    - v[iel][5]*8*pow(pi,2)
                    * cos(tpi * x)
                    * sin(tpi * y)
                    * cos(tpi * z);
      }
    }
    break;

    /*----------------------------------------------------*/

    case 6:
    for (cs_lnum_t iel = 0 ; iel < n_cells ; iel++) {
      f[iel] = 0.;
    }
    break;

    /*----------------------------------------------------*/

    /* K= 1*ID, T(x=0)=0, T(x=1)=1 */

    case 7:
    for (cs_lnum_t iel = 0 ; iel < n_cells ; iel++) {
      f[iel] = 0.;
    }
    break;

    /*----------------------------------------------------*/

    /*----------------------------------------------------*
     *   (1   0   0   )
     * K=(0   1   0   ), T(x,y,z) = x + y + z
     *   (0   0   1000)
     *----------------------------------------------------*/

    case 8:
    for (cs_lnum_t iel = 0 ; iel < n_cells ; iel++) {
      f[iel] = 0.;
    }
    break;

    /*----------------------------------------------------*/

    /*----------------------------------------------------*
     *   (y**2+1 -x*y    0)
     * K=(-x*y   x**2+1  0), T(x,y,z) = sin(Pi*x)*sin(Pi*y)
     *   (0        0     1)
     *----------------------------------------------------*/

    case 9:
    for (cs_lnum_t iel = 0 ; iel < n_cells ; iel++) {
      x = cell_cen[iel][0];
      y = cell_cen[iel][1];
      z = cell_cen[iel][2];

      sx = sin(pi*x);
      sy = sin(pi*y);

      cx = cos(pi*x);
      cy = cos(pi*y);

      solux = pi*cx*sy;
      soluy = pi*sx*cy;

      soluxx = -pow(pi,2)*sx*sy;
      soluyy = -pow(pi,2)*sx*sy;
      soluxy = pow(pi,2)*cx*cy;

      f[iel] = -((pow(y,2)+1.)*soluxx
               + (pow(x,2)+1.)*soluyy
               - 2.*x*y*soluxy
               - soluy*y - solux*x);
    }
    break;

    /*----------------------------------------------------*/

    /*----------------------------------------------------*
     *   (y**2+1   0     0)
     * K=(0        1     0), T(x,y,z) = sin(Pi*x)
     *   (0        0     1)
     *----------------------------------------------------*/

    case 10:
    for (cs_lnum_t iel = 0 ; iel < n_cells ; iel++) {
      x = cell_cen[iel][0];
      y = cell_cen[iel][1];
      z = cell_cen[iel][2];

      sx = sin(pi*x);

      cx = cos(pi*x);

      solux = pi*cx;

      soluxx = -pow(pi,2)*sx;

      f[iel] = -((pow(x,2)+1.)*soluxx - 2.*x*solux);
    }
    break;

    /*----------------------------------------------------*/

    /*----------------------------------------------------*
     *   (y**2+1   0     0)
     * K=(0      x**2+1  0), T(x,y,z) = sin(Pi*x)*sin(Pi*y)
     *   (0        0     1)
     *----------------------------------------------------*/

    case 11:
    for (cs_lnum_t iel = 0 ; iel < n_cells ; iel++) {
      x = cell_cen[iel][0];
      y = cell_cen[iel][1];
      z = cell_cen[iel][2];

      sx = sin(pi*x);
      sy = sin(pi*y);

      cx = cos(pi*x);
      cy = cos(pi*y);

      solux = pi*cx*sy;
      soluy = pi*sx*cy;

      soluxx = -pow(pi,2)*sx*sy;
      soluyy = -pow(pi,2)*sx*sy;
      soluxy = pow(pi,2)*cx*cy;

      f[iel] = -((pow(y,2)+1.)*soluxx
               + (pow(x,2)+1.)*soluyy);
    }
    break;

    /*----------------------------------------------------*/

    /*----------------------------------------------------*
     *   (y**2+z**2+1  -x*y      -x*z    )
     * K=(   -x*y   x**2+z**2+1  -y*z    ), T(x,y,z) = sin(Pi*x)*sin(Pi*y)*sin(Pi*z)
     *   (   -x*z      -y*z   x**2+y**2+1)
     *----------------------------------------------------*/

    case 12:
    for (cs_lnum_t iel = 0 ; iel < n_cells ; iel++) {
      x = cell_cen[iel][0];
      y = cell_cen[iel][1];
      z = cell_cen[iel][2];

      sx = sin(pi*x);
      sy = sin(pi*y);
      sz = sin(pi*z);

      cx = cos(pi*x);
      cy = cos(pi*y);
      cz = cos(pi*z);

      solux = pi*cx*sy*sz;
      soluy = pi*sx*cy*sz;
      soluz = pi*sx*sy*cz;

      soluxx = -pow(pi,2)*sx*sy*sz;
      soluyy = -pow(pi,2)*sx*sy*sz;
      soluzz = -pow(pi,2)*sx*sy*sz;

      soluxy = pow(pi,2)*cx*cy*sz;
      soluxz = pow(pi,2)*cx*sy*cz;
      soluyz = pow(pi,2)*sx*cy*cz;

      f[iel] =  - ((pow(y,2)+pow(z,2)+1.)*soluxx
                  +(pow(x,2)+pow(z,2)+1.)*soluyy
                  +(pow(x,2)+pow(y,2)+1.)*soluzz
                  - 2.*x*y*soluxy
                  - 2.*x*z*soluxz
                  - 2.*y*z*soluyz
                  - 2.*x*solux
                  - 2.*y*soluy
                  - 2.*z*soluz);
    }
    break;

    /*----------------------------------------------------*/

    default:
    for (cs_lnum_t iel = 0 ; iel < n_cells ; iel++) {
      f[iel] = -2*v[iel][2];
    }
    break;
 }
}

/*----------------------------------------------------------------------------*/
