/*----------------------------------------------------------------------------*/

/* VERS */

/*
  This file is part of Code_Saturne, a general-purpose CFD tool.

  Copyright (C) 1998-2014 EDF S.A.

  This program is free software; you can redistribute it and/or modify it under
  the terms of the GNU General Public License as published by the Free Software
  Foundation; either version 2 of the License, or (at your option) any later
  version.

  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
  details.

  You should have received a copy of the GNU General Public License along with
  this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
  Street, Fifth Floor, Boston, MA 02110-1301, USA.
*/

/*----------------------------------------------------------------------------*/

#include "cs_defs.h"

/*----------------------------------------------------------------------------
 * Standard C library headers
 *----------------------------------------------------------------------------*/

#include <assert.h>
#include <math.h>
#include <string.h>

#if defined(HAVE_MPI)
#include <mpi.h>
#endif

/*----------------------------------------------------------------------------
 * PLE library headers
 *----------------------------------------------------------------------------*/

#include <ple_coupling.h>

/*----------------------------------------------------------------------------
 *  Local headers
 *----------------------------------------------------------------------------*/

#include "bft_mem.h"
#include "bft_error.h"
#include "bft_printf.h"

#include "fvm_writer.h"

#include "cs_base.h"
#include "cs_field.h"
#include "cs_gui_util.h"
#include "cs_field_pointer.h"
#include "cs_field_operator.h"
#include "cs_math.h"
#include "cs_mesh.h"
#include "cs_mesh_location.h"
#include "cs_mesh_quantities.h"
#include "cs_halo.h"
#include "cs_halo_perio.h"
#include "cs_log.h"
#include "cs_multigrid.h"
#include "cs_parameters.h"
#include "cs_physical_constants.h"
#include "cs_prototypes.h"
#include "cs_sles.h"
#include "cs_sles_it.h"
#include "cs_time_moment.h"
#include "cs_time_step.h"
#include "cs_turbomachinery.h"
#include "cs_selector.h"

#include "cs_post.h"

/*----------------------------------------------------------------------------
 *  Header for the current file
 *----------------------------------------------------------------------------*/

#include "prototypes.h"

void
fvca_visc(cs_real_6_t v[],
          cs_real_t a[])
{

  const cs_lnum_t n_cells = cs_glob_mesh->n_cells;
  const cs_real_3_t *cell_cen = (const cs_real_3_t *)cs_glob_mesh_quantities->cell_cen;
  cs_real_t x, y, z;

/*----------------------------------------------------*/

  switch(testcase) {

    /*----------------------------------------------------*/
    case 1:
    for (cs_lnum_t iel = 0 ; iel < n_cells ; iel++) {
      v[iel][0] = 1.;
      v[iel][1] = 1.;
      v[iel][2] = 1.;
      v[iel][3] = 0.5;
      v[iel][4] = 0.5;
      v[iel][5] = 0.;
    }
    break;

    /*----------------------------------------------------*/

    case 2:
    for (cs_lnum_t iel = 0 ; iel < n_cells ; iel++) {
      x = cell_cen[iel][0];
      y = cell_cen[iel][1];
      z = cell_cen[iel][2];

      v[iel][0] = pow(y,2) + pow(z,2) + 1.;
      v[iel][1] = pow(x,2) + pow(z,2) + 1.;
      v[iel][2] = pow(x,2) + pow(y,2) + 1.;
      v[iel][3] = -x*y;
      v[iel][4] = -y*z;
      v[iel][5] = -x*z;
    }
    break;

    /*----------------------------------------------------*/

    case 3:
    for (cs_lnum_t iel = 0 ; iel < n_cells ; iel++) {
      v[iel][0] = 1.;
      v[iel][1] = 1.;
      v[iel][2] = 1000.;
      v[iel][3] = 0.;
      v[iel][4] = 0.;
      v[iel][5] = 0.;
    }
    break;

    /*----------------------------------------------------*/

    case 4:
    for (cs_lnum_t iel = 0 ; iel < n_cells ; iel++) {
      v[iel][0] = 1.;
      v[iel][1] = 1.;
      v[iel][2] = 0.2;
      v[iel][3] = 0.;
      v[iel][4] = 0.;
      v[iel][5] = 0.;
    }
    break;

    /*----------------------------------------------------*/

    case 5:
    for (cs_lnum_t iel = 0 ; iel < n_cells ; iel++) {
      a[0] = 0.;
      a[1] = 10.;
      a[2] = 100.;
      a[3] = 0.01;

      if (cell_cen[iel][1] <= 0.5 && cell_cen[iel][2] <= 0.5) {
        v[iel][0] = 1.;
        v[iel][1] = a[1];
        v[iel][2] = a[3];
        v[iel][3] = 0.;
        v[iel][4] = 0.;
        v[iel][5] = 0.;
      } else if (cell_cen[iel][1] > 0.5 && cell_cen[iel][2] <= 0.5) {
        v[iel][0] = 1.;
        v[iel][1] = a[0];
        v[iel][2] = a[2];
        v[iel][3] = 0.;
        v[iel][4] = 0.;
        v[iel][5] = 0.;
      } else if (cell_cen[iel][1] > 0.5 && cell_cen[iel][2] > 0.5) {
        v[iel][0] = 1.;
        v[iel][1] = a[3];
        v[iel][2] = a[1];
        v[iel][3] = 0.;
        v[iel][4] = 0.;
        v[iel][5] = 0.;
      } else {
        v[iel][0] = 1.;
        v[iel][1] = a[2];
        v[iel][2] = a[0];
        v[iel][3] = 0.;
        v[iel][4] = 0.;
        v[iel][5] = 0.;
      }
    }
    break;

    /*----------------------------------------------------*/

    /* K= 1*ID, T(x=0)=0, T(x=1)=1 */

    case 7:
    for (cs_lnum_t iel = 0 ; iel < n_cells ; iel++) {
      v[iel][0] = 1.;
      v[iel][1] = 1.;
      v[iel][2] = 1.;
      v[iel][3] = 0.;
      v[iel][4] = 0.;
      v[iel][5] = 0.;
    }
    break;

    /*----------------------------------------------------*/

    /*----------------------------------------------------*
     *   (1   0   0   )
     * K=(0   1   0   ), T(x,y,z) = x + y + z
     *   (0   0   1000)
     *----------------------------------------------------*/

    case 8:
    for (cs_lnum_t iel = 0 ; iel < n_cells ; iel++) {
      v[iel][0] = 1.;
      v[iel][1] = 1.;
      v[iel][2] = 1000.;
      v[iel][3] = 0.;
      v[iel][4] = 0.;
      v[iel][5] = 0.;
    }
    break;

    /*----------------------------------------------------*/

    /*----------------------------------------------------*
     *   (y**2+1 -x*y    0)
     * K=(-x*y   x**2+1  0), T(x,y,z) = sin(Pi*x)*sin(Pi*y)
     *   (0        0     1)
     *----------------------------------------------------*/

    case 9:
    for (cs_lnum_t iel = 0 ; iel < n_cells ; iel++) {
      x = cell_cen[iel][0];
      y = cell_cen[iel][1];
      z = cell_cen[iel][2];

      v[iel][0] = 1.*pow(y,2);
      v[iel][1] = 1.+pow(x,2);
      v[iel][2] = 1.+pow(x,2);
      v[iel][3] = -x*y;
      v[iel][4] = 0.;
      v[iel][5] = 0.;
    }
    break;

    /*----------------------------------------------------*/

    /*----------------------------------------------------*
     *   (y**2+1   0     0)
     * K=(0        1     0), T(x,y,z) = sin(Pi*x)
     *   (0        0     1)
     *----------------------------------------------------*/

    case 10:
    for (cs_lnum_t iel = 0 ; iel < n_cells ; iel++) {
      v[iel][0] = 1. + pow(cell_cen[iel][0],2);
      v[iel][1] = 1.;
      v[iel][2] = 1.;
      v[iel][3] = 0.;
      v[iel][4] = 0.;
      v[iel][5] = 0.;
    }
    break;

    /*----------------------------------------------------*/

    /*----------------------------------------------------*
     *   (y**2+1   0     0)
     * K=(0      x**2+1  0), T(x,y,z) = sin(Pi*x)*sin(Pi*y)
     *   (0        0     1)
     *----------------------------------------------------*/

    case 11:
    for (cs_lnum_t iel = 0 ; iel < n_cells ; iel++) {
      v[iel][0] = 1. + pow(cell_cen[iel][1],2);
      v[iel][1] = 1. + pow(cell_cen[iel][0],2);
      v[iel][2] = 1.;
      v[iel][3] = 0.;
      v[iel][4] = 0.;
      v[iel][5] = 0.;
    }
    break;

    /*----------------------------------------------------*/

    /*----------------------------------------------------*
     *   (y**2+z**2+1  -x*y      -x*z    )
     * K=(   -x*y   x**2+z**2+1  -y*z    ), T(x,y,z) = sin(Pi*x)*sin(Pi*y)*sin(Pi*z)
     *   (   -x*z      -y*z   x**2+y**2+1)
     *----------------------------------------------------*/

    case 12:
    for (cs_lnum_t iel = 0 ; iel < n_cells ; iel++) {
      x = cell_cen[iel][0];
      y = cell_cen[iel][1];
      z = cell_cen[iel][2];

      v[iel][0] = pow(y,2) + pow(z,2) + 1.;
      v[iel][1] = pow(x,2) + pow(z,2) + 1.;
      v[iel][2] = pow(x,2) + pow(y,2) + 1.;
      v[iel][3] = -x*y;
      v[iel][4] = -y*z;
      v[iel][5] = -x*z;
    }
    break;

    /*----------------------------------------------------*/

    default:
    for (cs_lnum_t iel = 0 ; iel < n_cells ; iel++) {
      v[iel][0] = 1.;
      v[iel][1] = 1.;
      v[iel][2] = 1000.;
      v[iel][3] = 0.;
      v[iel][4] = 0.;
      v[iel][5] = 0.;
    }
    break;
 }
}

/*----------------------------------------------------------------------------*/
