/*============================================================================
 * This function is called at the end of each time step, and has a very
 *  general purpose
 *  (i.e. anything that does not have another dedicated user subroutine)
 *============================================================================*/

/* VERS */

/*
  This file is part of Code_Saturne, a general-purpose CFD tool.

  Copyright (C) 1998-2016 EDF S.A.

  This program is free software; you can redistribute it and/or modify it under
  the terms of the GNU General Public License as published by the Free Software
  Foundation; either version 2 of the License, or (at your option) any later
  version.

  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
  details.

  You should have received a copy of the GNU General Public License along with
  this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
  Street, Fifth Floor, Boston, MA 02110-1301, USA.
*/

/*----------------------------------------------------------------------------*/

#include "cs_defs.h"

/*----------------------------------------------------------------------------
 * Standard C library headers
 *----------------------------------------------------------------------------*/

#include <assert.h>
#include <math.h>

#if defined(HAVE_MPI)
#include <mpi.h>
#endif

/*----------------------------------------------------------------------------
 * PLE library headers
 *----------------------------------------------------------------------------*/

#include <ple_coupling.h>

/*----------------------------------------------------------------------------
 *  Local headers
 *----------------------------------------------------------------------------*/

#include "bft_mem.h"
#include "bft_error.h"
#include "bft_printf.h"

#include "fvm_writer.h"

#include "cs_base.h"
#include "cs_blas.h"
#include "cs_field.h"
#include "cs_field_pointer.h"
#include "cs_field_operator.h"
#include "cs_mesh.h"
#include "cs_mesh_quantities.h"
#include "cs_halo.h"
#include "cs_halo_perio.h"
#include "cs_log.h"
#include "cs_parameters.h"
#include "cs_prototypes.h"
#include "cs_rotation.h"
#include "cs_time_moment.h"
#include "cs_time_step.h"
#include "cs_turbomachinery.h"
#include "cs_selector.h"

#include "cs_post.h"

/*----------------------------------------------------------------------------
 *  Header for the current file
 *----------------------------------------------------------------------------*/

#include "cs_prototypes.h"
#include "prototypes.h"

/*----------------------------------------------------------------------------*/

BEGIN_C_DECLS

/*----------------------------------------------------------------------------*/
/*!
 * \file cs_user_extra_operations.c
 *
 * \brief This function is called at the end of each time step, and has a very
 * general purpose (i.e. anything that does not have another dedicated
 * user subroutine)
 *
 */
/*----------------------------------------------------------------------------*/

/*============================================================================
 * User function definitions
 *============================================================================*/

/*----------------------------------------------------------------------------*/
/*!
 * \brief This function is called at the end of each time step.
 *
 * It has a very general purpose, although it is recommended to handle
 * mainly postprocessing or data-extraction type operations.
 */
/*----------------------------------------------------------------------------*/

void
cs_user_extra_operations(cs_domain_t     *domain)
{
  const cs_lnum_t n_cells     = cs_glob_mesh->n_cells;
  const cs_lnum_t n_cells_ext = cs_glob_mesh->n_cells_with_ghosts;
  const cs_lnum_t n_faces     = cs_glob_mesh->n_i_faces;
  const cs_real_t *volume = cs_glob_mesh_quantities->cell_vol;
  cs_real_t *er1, *er2, *ern1, *ern2, *erg11, *erg12, *erg13, *erg21, *erg22, *erg23;
  cs_real_t erl2, ergrad, umax, umin, uemax, uemin, proder1, proder2;
  cs_real_t prodergx1, prodergx2, prodergy1, prodergy2, prodergz1, prodergz2;
  cs_real_t prodern1, prodern2, ener, *sol, alpha_i[4];
  cs_real_3_t *grad, *gradsol;
  int nmat;
  static FILE *f1 = NULL, *f2 = NULL;

  /*===============================================================================
   * 1.  Initialization
   *===============================================================================*/

  cs_field_t *fturbvisco = cs_field_by_name_try("anisotropic_turbulent_viscosity");
  cs_real_6_t *visten = (cs_real_6_t *)fturbvisco->val;

  BFT_MALLOC(er1, n_cells_ext, cs_real_t);
  BFT_MALLOC(er2, n_cells_ext, cs_real_t);
  BFT_MALLOC(erg11, n_cells_ext, cs_real_t);
  BFT_MALLOC(erg12, n_cells_ext, cs_real_t);
  BFT_MALLOC(erg13, n_cells_ext, cs_real_t);
  BFT_MALLOC(erg21, n_cells_ext, cs_real_t);
  BFT_MALLOC(erg22, n_cells_ext, cs_real_t);
  BFT_MALLOC(erg23, n_cells_ext, cs_real_t);
  BFT_MALLOC(ern1, n_cells_ext, cs_real_t);
  BFT_MALLOC(ern2, n_cells_ext, cs_real_t);
  BFT_MALLOC(sol, n_cells_ext, cs_real_t);
  BFT_MALLOC(grad, n_cells_ext, cs_real_3_t);
  BFT_MALLOC(gradsol, n_cells_ext, cs_real_3_t);

  if (cs_glob_time_step->nt_cur == 1) {
    f1 = fopen("Bilan.dat","a");
    f2 = fopen("VnVOutput.dat","a");
  }

  erl2 = 0.;
  ergrad = 0.;
  nmat = 0.;
  umax=  0.;
  umin = 0.;
  uemin = 0.;
  uemax = 0.;

  fvca_visc (visten, alpha_i);

  fvca_sol(sol, alpha_i);

  fvca_grad(gradsol, alpha_i);

  fvca_nmat(&nmat, visten);

  int keysca = cs_field_key_id("scalar_id");
  cs_field_t *sca1 = NULL;

  for (int f_id = 0; f_id < cs_field_n_fields(); f_id++) {
    cs_field_t *f = cs_field_by_id(f_id);
    if (cs_field_get_key_int(f, keysca) == 1) {
      sca1 = f;
      break;
    }
  }

  cs_field_gradient_scalar(sca1,
                           false, /* use_previous_t */
                           1,    /* inc */
                           true, /* recompute_cocg */
                           grad);

  for (cs_lnum_t iel  = 0 ; iel < n_cells ; iel++) {
    er1[iel] = (sol[iel]-sca1->val[iel]);
    er2[iel] = sol[iel];

    erg11[iel] = (gradsol[iel][0]-grad[iel][0]);
    erg12[iel] = (gradsol[iel][1]-grad[iel][1]);
    erg13[iel] = (gradsol[iel][2]-grad[iel][2]);

    erg21[iel] = gradsol[iel][0];
    erg22[iel] = gradsol[iel][1];
    erg23[iel] = gradsol[iel][2];

    ern1[iel] = visten[iel][0]*pow(gradsol[iel][0] - grad[iel][0],2)
              + visten[iel][1]*pow(gradsol[iel][1] - grad[iel][1],2)
              + visten[iel][2]*pow(gradsol[iel][2] - grad[iel][2],2)
              + 2.*visten[iel][3]*(gradsol[iel][0] - grad[iel][0])
                                 *(gradsol[iel][1] - grad[iel][1])
              + 2.*visten[iel][4]*(gradsol[iel][1] - grad[iel][1])
                                 *(gradsol[iel][2] - grad[iel][2])
              + 2.*visten[iel][5]*(gradsol[iel][0] - grad[iel][0])
                                 *(gradsol[iel][2] - grad[iel][2]);

    ern2[iel] = visten[iel][0]*pow(gradsol[iel][0],2)
              + visten[iel][1]*pow(gradsol[iel][1],2)
              + visten[iel][2]*pow(gradsol[iel][2],2)
              + 2.*visten[iel][3]*gradsol[iel][0]*gradsol[iel][1]
              + 2.*visten[iel][4]*gradsol[iel][1]*gradsol[iel][2]
              + 2.*visten[iel][5]*gradsol[iel][0]*gradsol[iel][2];
  }

  proder1 = sqrt(cs_gres(n_cells, volume, er1, er1));
  proder2 = sqrt(cs_gres(n_cells, volume, er2, er2));

  prodergx1 = cs_gres(n_cells, volume, erg11, erg11);
  prodergy1 = cs_gres(n_cells, volume, erg12, erg12);
  prodergz1 = cs_gres(n_cells, volume, erg13, erg13);
  prodergx2 = cs_gres(n_cells, volume, erg21, erg21);
  prodergy2 = cs_gres(n_cells, volume, erg22, erg22);
  prodergz2 = cs_gres(n_cells, volume, erg23, erg23);

  /* Warning this is an L4 norm */
  prodern1 = sqrt(cs_gdot(n_cells, ern1, ern1));
  prodern2 = sqrt(cs_gdot(n_cells, ern2, ern2));

  erl2 = proder1 / proder2;
  ergrad= sqrt((prodergx1+prodergy1+prodergz1)/(prodergx2+prodergy2+prodergz2));
  ener = sqrt(prodern1 / prodern2);

  for (cs_lnum_t iel  = 0 ; iel < n_cells ; iel++) {
    umax = CS_MAX(umax, sca1->val[iel]);
    uemin = CS_MIN(umin, sca1->val[iel]);
    uemax = CS_MAX(uemax, sol[iel]);
    uemin = CS_MIN(uemin, sol[iel]);
  }

  if (cs_glob_time_step->nt_cur == cs_glob_time_step->nt_max) {
    const cs_solving_info_t *sinfo
      = cs_field_get_key_struct_const_ptr(sca1, cs_field_key_id("solving_info"));
    const cs_var_cal_opt_t *var_cal_opt
      = cs_field_get_key_struct_const_ptr(sca1, cs_field_key_id("var_cal_opt"));

    fprintf(f1,"Testcase: %10d\n",testcase);
    fprintf(f1,"Viscosité 1: %12.1e %12.1e %12.1e\n",
            visten[0][0], visten[0][3], visten[0][5]);
    fprintf(f1,"Viscosité 2: %12.1e %12.1e %12.1e\n",
            visten[0][3], visten[0][1], visten[0][4]);
    fprintf(f1,"Viscosité 3: %12.1e %12.1e %12.1e\n",
            visten[0][5], visten[0][4], visten[0][2]);
    fprintf(f1,"Reconstruction: %10d\n",var_cal_opt->imrgra);
    fprintf(f1,"Relaxation: %14.4f\n", var_cal_opt->relaxv);
    fprintf(f1,"nu: %10d\n",n_cells);
    fprintf(f1,"nmat: %10d\n",nmat);
    fprintf(f1,"uemin: %12.4e\n",uemin);
    fprintf(f1,"umax: %12.4e\n",umax);
    fprintf(f1,"uemax: %12.4e\n",uemax);
    fprintf(f1,"erl2: %12.4e\n",erl2);
    fprintf(f1,"ergrad: %12.4e\n",ergrad);
    fprintf(f1,"ener: %12.4e\n",ener);
    fprintf(f2,"# Test case, nu, erl2, ergrad, ener, N_iter, ncel, nfac, ncel+nfac, cost\n");
    fprintf(f2,"%2d %10d %17.9e %17.9e %17.9e %10d %10d %10d %10d %17.9e\n",
               testcase, n_cells, erl2, ergrad, ener, sinfo->n_it, n_cells, n_faces,
               n_cells+n_faces, (double)sinfo->n_it*(n_cells+n_faces));
    fclose(f1);
    fclose(f2);
  }

  BFT_FREE(er1);
  BFT_FREE(er2);
  BFT_FREE(erg11);
  BFT_FREE(erg12);
  BFT_FREE(erg13);
  BFT_FREE(erg21);
  BFT_FREE(erg22);
  BFT_FREE(erg23);
  BFT_FREE(ern1);
  BFT_FREE(ern2);
  BFT_FREE(grad);
  BFT_FREE(gradsol);
  BFT_FREE(sol);
}

END_C_DECLS
