/*============================================================================
 * User subroutines for input of calculation parameters.
 *============================================================================*/

/* VERS */

/*
  This file is part of Code_Saturne, a general-purpose CFD tool.

  Copyright (C) 1998-2014 EDF S.A.

  This program is free software; you can redistribute it and/or modify it under
  the terms of the GNU General Public License as published by the Free Software
  Foundation; either version 2 of the License, or (at your option) any later
  version.

  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
  details.

  You should have received a copy of the GNU General Public License along with
  this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
  Street, Fifth Floor, Boston, MA 02110-1301, USA.
*/

/*----------------------------------------------------------------------------*/

#include "cs_defs.h"

/*----------------------------------------------------------------------------
 * Standard C library headers
 *----------------------------------------------------------------------------*/

#include <assert.h>
#include <math.h>
#include <string.h>

#if defined(HAVE_MPI)
#include <mpi.h>
#endif

/*----------------------------------------------------------------------------
 * PLE library headers
 *----------------------------------------------------------------------------*/

#include <ple_coupling.h>

/*----------------------------------------------------------------------------
 *  Local headers
 *----------------------------------------------------------------------------*/

#include "bft_mem.h"
#include "bft_error.h"
#include "bft_printf.h"

#include "fvm_writer.h"

#include "cs_base.h"
#include "cs_convection_diffusion.h"
#include "cs_field.h"
#include "cs_gui_util.h"
#include "cs_field_pointer.h"
#include "cs_field_operator.h"
#include "cs_math.h"
#include "cs_mesh.h"
#include "cs_mesh_location.h"
#include "cs_mesh_quantities.h"
#include "cs_halo.h"
#include "cs_halo_perio.h"
#include "cs_log.h"
#include "cs_multigrid.h"
#include "cs_parameters.h"
#include "cs_physical_constants.h"
#include "cs_prototypes.h"
#include "cs_sles.h"
#include "cs_sles_it.h"
#include "cs_time_moment.h"
#include "cs_time_step.h"
#include "cs_turbomachinery.h"
#include "cs_turbulence_model.h"
#include "cs_selector.h"

#include "cs_post.h"

/*----------------------------------------------------------------------------
 *  Header for the current file
 *----------------------------------------------------------------------------*/

#include "cs_prototypes.h"
#include "prototypes.h"

/*----------------------------------------------------------------------------*/

BEGIN_C_DECLS

/*============================================================================
 * User function definitions
 *============================================================================*/

int testcase = 3;

/*----------------------------------------------------------------------------*/
/*!
 * \brief Select physical model options, including user fields.
 *
 * This function is called at the earliest stages of the data setup,
 * so field ids are not available yet.
 */
/*----------------------------------------------------------------------------*/

void
cs_user_model(void)
{
  cs_parameters_add_variable("scalar_1", 1);

  cs_turb_model_t *tm = cs_get_glob_turb_model();
  tm->iturb = 0;
}

/*----------------------------------------------------------------------------*/
/*!
 * \brief Define or modify general numerical and physical user parameters.
 *
 * At the calling point of this function, most model-related most variables
 * and other fields have been defined, so specific settings related to those
 * fields may be set here.
 */
/*----------------------------------------------------------------------------*/

void
cs_user_parameters(cs_domain_t     *domain)
{
  cs_time_step_t *ts = cs_get_glob_time_step();
  cs_time_step_options_t *tso = cs_get_glob_time_step_options();
  cs_fluid_properties_t *fp = cs_get_glob_fluid_properties();
  cs_space_disc_t *sd = cs_get_glob_space_disc();
  int key_cal_opt_id = cs_field_key_id("var_cal_opt");
  cs_var_cal_opt_t var_cal_opt;
  int keysca = cs_field_key_id("scalar_id");
  cs_field_t *sca1 = NULL;

  for (int f_id = 0; f_id < cs_field_n_fields(); f_id++) {
    cs_field_t *f = cs_field_by_id(f_id);
    if (cs_field_get_key_int(f, keysca) == 1) {
      sca1 = f;
      break;
    }
  }

  cs_field_get_key_struct(sca1, key_cal_opt_id, &var_cal_opt);
  var_cal_opt.iwarni = 5;
  var_cal_opt.iconv = 0;
  var_cal_opt.idften = CS_ANISOTROPIC_DIFFUSION;
  var_cal_opt.idifft = 1;
  var_cal_opt.epsrgr = 1.e-14;
  var_cal_opt.epsrsm = 1.e-10;
  var_cal_opt.epsilo = 1.e-15;
  var_cal_opt.nswrgr = 200;
  var_cal_opt.nswrsm = 200;
  var_cal_opt.istat = 0;
  cs_field_set_key_struct(sca1, key_cal_opt_id, &var_cal_opt);

  for (int f_id = 0 ; f_id < cs_field_n_fields() ; f_id++) {
    cs_field_t *f = cs_field_by_id(f_id);
    if (f->type & CS_FIELD_VARIABLE) {
      cs_field_get_key_struct(f, key_cal_opt_id, &var_cal_opt);
      var_cal_opt.iswdyn = 1.;
      cs_field_set_key_struct(f, key_cal_opt_id, &var_cal_opt);
    }
  }

  ts->nt_max = 1;
  ts->dt_ref = 0.01;
  sd->imrgra = 0;
  sd->imvisf = 1;

  fp->ro0 = 1.;
  fp->viscl0 = 1.;
  fp->cp0 = 1.;
  fp->t0 = 1000.+273.15;
  fp->p0 = 1.01325e5;
}

/*----------------------------------------------------------------------------*/
/*!
 * \brief Define linear solver options.
 *
 * This function is called at the setup stage, once user and most model-based
 * fields are defined.
 */
/*----------------------------------------------------------------------------*/

void
cs_user_linear_solvers(void)
{
  cs_sles_it_define(CS_F_(vel)->id, NULL, CS_SLES_BICGSTAB, 0, 10000);

}

/*----------------------------------------------------------------------------*/

END_C_DECLS
